#!/usr/bin/env python
#Copyright Bob Blair, 2010-2012
#Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, distribute with modifications, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
#The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#Except as contained in this notice, the name(s) of the above copyright holders shall not be used in advertising or otherwise to promote the sale, use or other dealings in this Software without prior written authorization.
#

# Generate the main indexes, which have entries withing subjects.
# The idea is that we can have a subject index that points into these files
# for the detailed entries.  We'll try to keep these index files in the
# range of 250K in size.

# Change log.
#   1/21/2012  Add document type and css link

import sys
import re
from indexutil import slist, getURL, roman2int

def set_headstrs():
  strs = []
  strs.append('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"')
  strs.append('        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">')
  strs.append('<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">')
  strs.append('<head><title>')
  strs.append('Index of Samuel R. Gardiner\'s History of England')
  strs.append('</title>')
  strs.append('<meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>')
  strs.append('<link href="index.css" rel="stylesheet" type="text/css"/>')
  strs.append('</head><body>')
  strs.append('<h1 align=\"center\">')
  strs.append('Index of Gardiner\'s History of England')
  strs.append('</h1><ul>')
  return strs

def set_tailstrs():
  strs = []
  strs.append('</ul></li></ul></body></html>')
  return strs

def tell_usage():
  sys.stderr.write('This script generates a monolithic index of Gardiner\'s\n')
  sys.stderr.write('History of England by reading and processing a text file\n')
  sys.stderr.write('that contains the index data.  You must specify the name\n')
  sys.stderr.write('of the text file as the first command line argument.\n')
  return

def generate_html(index):
  #initialize
  subject = ""
  subjectcnt = 0
  lastvol = 0 # lastvol is used to process entries that don't have a volume
  lastpage = 0 # lastpage is used to process ibid. entries
  # print the html head
  print ''.join(set_headstrs());
  # create a list of compiled regular expressions for parsing out pieces
  # of the input file
  plist = [];
  for s in slist:
    plist.append(re.compile(s))
  # parse each line in the input
  for line in index:
    # skip blank lines (slist[5] matches a blank line)
    m = re.match(plist[5], line) # check blank line
    if (m):
      continue
    # Process a new subject
    m = re.match(plist[0], line) # check start of subject
    if (m):
      if subjectcnt > 0:
        print '</ul></li>'
      s = '<li><a name=\"S' + str(subjectcnt) + '\">' + line + '</a><ul>' 
      print s
      subject = line
      subjectcnt = subjectcnt + 1
      lastvol = 0
      lastpage = 0
      continue
    # Process a reference
    m = re.match(plist[1], line) # check "See xxx"
    if m:
      print '<li>', m.group(0), '</li>'
      continue
    # Process an "ibid."
    m = re.match(plist[2], line) # check "xxx ib."
    if m:
      s = '<li><a href=\"' + getURL(lastvol, lastpage) + '\">'
      s += m.group(1) +  '</a></li>'
      print s
      continue
    # process a normal index entry under a subject, with a volume specified
    m = re.match(plist[3], line) # check "yada yada iii. 99"
    if m:
      vol = roman2int(m.group(2))
      page = int(m.group(3))
      s = '<li><a href=\"' + getURL(vol, page) + '\">'
      s += m.group(1) +  '</a></li>'
      print s
      lastvol = vol
      lastpage = page
      continue
 # process a normal index entry under a subject, without a volume specified
    m = re.match(plist[4], line) # check xxxx 99
    if (m):
      page = int(m.group(2))
      s = '<li><a href=\"' + getURL(lastvol, page) + '\">'
      s += m.group(1) +  '</a></li>'
      print s
      lastpage = page
      continue
  print ''.join(set_tailstrs())
  return

def main():
  # open the input file and suck the whole thing in and print the html document
  if len(sys.argv) > 1 and sys.argv[1] != '?':
    f = open(sys.argv[1])
    index = f.readlines();
    f.close()
    generate_html(index)
  else:
    tell_usage()

if __name__ == '__main__':
  main()
