#!/usr/bin/env python
#Copyright Bob Blair, 2010-2012
#Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, distribute with modifications, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
#The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#Except as contained in this notice, the name(s) of the above copyright holders shall not be used in advertising or otherwise to promote the sale, use or other dealings in this Software without prior written authorization.
#

# Generate a subject index (much smaller than the monolithic entry index)
# that points to the detailed entries for each subject.  In this version,
# the subjects pointed to are in seperate files, as generates by 
# indexreports.py

import sys
import re
from indexutil import slist, getURL, roman2int

def create_href(outdir, subjectno):
    return outdir + '/' + 'S' + str(subjectno) + '.html'

def set_headstrs():
    strs = []
    strs.append('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"')
    strs.append('        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">')
    strs.append('<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">')
    strs.append('<head><title>')
    strs.append('Subject Index of Samuel R. Gardiner\'s History of England')
    strs.append('</title>')
    strs.append('<meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>')
    strs.append('<link href="index.css" rel="stylesheet" type="text/css"/>')
    strs.append('</head><body>')
    strs.append('<h1 align=\"center\">')
    strs.append('Index of Subjects in Gardiner\'s History of England</h1><ul>')
    return strs

def set_bodystrs():
    strs = []
    strs.append('<li><a href=\"')
    strs.append('') # bodystrs[1] will be the target href
    hrefpos = 1
    strs.append('\">')
    strs.append('') # bodystrs[3] will be the subject line
    subjlinepos = 3
    strs.append('</a></li>')
    return strs, hrefpos, subjlinepos

def set_tailstrs():
    strs = []
    strs.append('</ul></body></html>')
    return strs

def generate_html(index, outdir):
    # Create the output string arrays.
    headstrs = set_headstrs()
    bodystrs, hrefpos, subjlinepos = set_bodystrs()
    tailstrs = set_tailstrs()
    # Compile the regular expressions we'll need.
    plist = [];
    for regex in slist:
        plist.append(re.compile(regex))
    # print the html header
    print ''.join(headstrs)
    #print each subject line (the regex for a subject line is in plist[0])
    subjectcnt = 0
    for line in index:
        match = re.match(plist[0], line) # check start of subject
        if (match):
            bodystrs[hrefpos] = create_href(outdir, str(subjectcnt))
            bodystrs[subjlinepos] = line.strip().rstrip('.,')
            print ''.join(bodystrs)
            subjectcnt += 1
    # print the html tail
    print ''.join(tailstrs)

def tell_usage():
  sys.stderr.write('This script generates a subject index of Gardiner\'s\n')
  sys.stderr.write('History of England by reading and processing a text file\n')
  sys.stderr.write('that contains the index data.  You must specify the name\n')
  sys.stderr.write('of the text file as the first command line argument.\n')
  sys.stderr.write('Each subject entry points to a list of references to the\n')
  sys.stderr.write('subject in Gardiner\'s history.  In particular, this\n')
  sys.stderr.write('index points into files generated by indexreports.py\n\n')
  sys.stderr.write('If you want this subject index to be in a directory\n')
  sys.stderr.write('other than the directory containing the index reports,\n')
  sys.stderr.write('specify the relative path to the index reports as the\n')
  sys.stderr.write('(optional) second command line parameter.\n')
  return

def main():
    # open the input file and suck the whole thing in and print the html doc
    if len(sys.argv) > 1 and sys.argv[1] != '?':
        f = open(sys.argv[1])
        index = f.readlines();
        f.close()
        outdir = '.'
        if len(sys.argv) > 2:
            outdir = sys.argv[2]
        generate_html(index, outdir)
    else:
        tell_usage()


if __name__ == "__main__":
    main()
